/*

------------------------------------------------------------------------------

A license is hereby granted to reproduce this software source code and
to create executable versions from this source code for personal,
non-commercial use.  The copyright notice included with the software
must be maintained in all copies produced.

THIS PROGRAM IS PROVIDED "AS IS". THE AUTHOR PROVIDES NO WARRANTIES
WHATSOEVER, EXPRESSED OR IMPLIED, INCLUDING WARRANTIES OF
MERCHANTABILITY, TITLE, OR FITNESS FOR ANY PARTICULAR PURPOSE.  THE
AUTHOR DOES NOT WARRANT THAT USE OF THIS PROGRAM DOES NOT INFRINGE THE
INTELLECTUAL PROPERTY RIGHTS OF ANY THIRD PARTY IN ANY COUNTRY.

Copyright (c) 1995, John Conover, All Rights Reserved.

Comments and/or bug reports should be addressed to:

    john@johncon.com (John Conover)

------------------------------------------------------------------------------

translit.c, transliterate a page

ssize_t transliterate (unsigned char *page, ssize_t count);

    translate count many characters in page using uppercase as a
    translation table

    note: the sole reason for breaking this module out of searchfile()
    is to provide a means of manipulating the content of a file that
    is being searched-the rules are:

        1) the search area must start at page[0], (but can constitute
        a smaller area of the page data space,) and the search area
        must end a ' ' character; it is a requirement of bmhsearch(),
        in bmhsearch.c, that the '\0' character is reserved as an end
        of search sentinel in the pattern-failure to observe this rule
        will result in a program that is erratic and either hangs
        forever, or perhaps does a core dump of a very involved data
        structure, that is very difficult to analyze-see also
        uppercase.c and bmhsearch.c

        2) the return value, count, must be the size of the data
        space, in page to be searched, *_NOT_* including the last ' '
        character

As an enhancement, in conjunction with uppercase.c, (albeit at the
expense of performance,) hyphenation, backspace and underlining, and
phrase searching could be addressed:

    1) hyphenation could be implemented by omitting a '-' followed by
    any number of white space characters

    2) if the program is used primarily for searching catman pages,
    the backspace and underlining features that are incorporated in
    the man page system can be defeated by deleting the "backspace
    character" sequences from the documents.

    3) phrase searching could be enhanced by translating any number of
    whitespace characters into a single ' ' character-the "\ " search
    phrase would then be interpreted as any number of white space
    characters

    Note that main() in rel.c calls transliterate() in tranlit.c to
    transliterate the query/search criteria-if an exact match is
    specified, both the pattern and the data would be altered in
    exactly the same manner, and appropriate matches found even though
    both were translated, (although additional patterns could
    conceivably be matched, the originals will be found,
    irregardless,) for example, the data:

        ... re-engineering ...

    or hyphenated:

        ... re-
        engineering ...

    would become reengineering, which could be found by any of the
    query patterns:

        re
        engineering
        reengineering
        re-engineering

    Likewise for multiple space compression in phrase query patterns.
    Quite probably, such scenarios should be controlled by command
    line options, perhaps via a language selection to avoid
    localization and portability conflicts.

The algorithm is as follows:

    for each character in the page

        replace the character with its equivilent in uppercase[]

Usage is a call with page referencing the first character to be
translated, and count the number of characters to be translated,
for example:

    count = transliterate (page, count + 2);

There are no errors, and the number of characters translated is
returned

To test this module, compile the module source with -DTEST_TRANSLIT

$Revision: 1.0 $
$Date: 1995/04/22 05:13:18 $
$Id: translit.c,v 1.0 1995/04/22 05:13:18 john Exp $
$Log: translit.c,v $
 * Revision 1.0  1995/04/22  05:13:18  john
 * Initial revision
 *

*/

#include "rel.h"

#ifndef LINT /* include rcsid only if not running lint */

static char rcsid[] = "$Id: translit.c,v 1.0 1995/04/22 05:13:18 john Exp $"; /* module version */
static char rcsid_h[] = TRANSLIT_H_ID; /* module include version */

#endif

#ifdef __STDC__

ssize_t transliterate (unsigned char *page, ssize_t count)

#else

ssize_t transliterate (page, count)
    unsigned char *page;
    ssize_t count;

#endif

{
    unsigned char *char_ref = page; /* reference to character in memory page */

    int i; /* character counter */

    for (i = 0; i < count; i++) /* for each character in the page */
    {
        *char_ref = uppercase[(int) *char_ref]; /* convert the character to uppercase */
        char_ref++; /* next character */
    }

    return (count); /* return the size of the page */
}

#ifdef TEST_TRANSLIT

/*

simple exerciser for testing transliterate (); get a string from
stdin, transliterate it, and print it to stdout; ignore the:

declared global, could be static
    transliterate       translit.c(xx)

from lint

*/

#ifdef __STDC__

int main (void)

#else

int main ()

#endif

{
    unsigned char buffer[BUFSIZ]; /* buffer to be parsed */

    if (make_uppercase () != (unsigned char *) 0) /* setup the uppercase array */
    {

        while (gets ((char *) buffer) != 0) /* input the string to be transliterated */
        {
            (void) transliterate (buffer, strlen ((char *) buffer)); /* transliterate the buffer */
            (void) printf ("%s\n", buffer); /* print the transliterate buffer */
        }

    }

    else
    {
        (void) fprintf (stderr, "error making uppercase array\n"); /* couldn't setup the uppercase array, print the error */
        exit (1); /* and exit */
    }

    exit (0); /* return success */

#ifdef LINT /* include only if running lint */

    return (0); /* for LINT formality */

#endif

}

#endif
